import importlib

from django.apps import AppConfig as BaseAppConfig
from django.utils.translation import ugettext_lazy as _


class AppConfig(BaseAppConfig):

    name = "pinax.documents"
    verbose_name = _("Pinax Documents")

    def ready(self):
        importlib.import_module("pinax.documents.receivers")



import math


def convert_bytes(bytes):
    bytes = float(bytes)
    if bytes >= 1099511627776:
        size, srepr = bytes / 1099511627776, "TB"
    elif bytes >= 1073741824:
        size, srepr = bytes / 1073741824, "GB"
    elif bytes >= 1048576:
        size, srepr = bytes / 1048576, "MB"
    elif bytes >= 1024:
        size, srepr = bytes / 1024, "KB"
    else:
        size, srepr = bytes, " bytes"
    return "%d%s" % (math.ceil(size), srepr)



import itertools
import operator

from django.apps import apps
from django.db import models
from django.db.models import Q
from django.db.models.query import QuerySet


class FolderManager(models.Manager):

    def members(self, folder, **kwargs):
        direct = kwargs.get("direct", True)
        user = kwargs.get("user")
        Document = apps.get_model("documents", "Document")
        folders = self.filter(parent=folder)
        documents = Document.objects.filter(folder=folder)
        if user:
            folders = folders.for_user(user)
            documents = documents.for_user(user)
        M = sorted(itertools.chain(folders, documents), key=operator.attrgetter("name"))
        if direct:
            return M
        for child in folders:
            M.extend(self.members(child, **kwargs))
        return M


class FolderQuerySet(QuerySet):

    def for_user(self, user):
        """
        All folders the given user can do something with.
        """
        qs = SharedMemberQuerySet(model=self.model, using=self._db, user=user)
        qs = qs.filter(Q(author=user) | Q(foldershareduser__user=user))
        return qs.distinct() & self.distinct()


class DocumentQuerySet(QuerySet):

    def for_user(self, user):
        """
        All documents the given user can do something with.
        """
        qs = SharedMemberQuerySet(model=self.model, using=self._db, user=user)
        qs = qs.filter(Q(author=user) | Q(documentshareduser__user=user))
        return qs.distinct() & self.distinct()


class SharedMemberQuerySet(QuerySet):

    def __init__(self, **kwargs):
        if "user" in kwargs:
            self.user = kwargs.pop("user")
        super(SharedMemberQuerySet, self).__init__(**kwargs)

    def iterator(self):
        shared_user_model = self.model.shared_user_model()
        shared_members = shared_user_model.for_user(self.user)
        for obj in super(SharedMemberQuerySet, self).iterator():
            if obj.pk in shared_members:
                obj._shared = True
            yield obj

    def _clone(self, **kwargs):
        kwargs["user"] = self.user
        return super(SharedMemberQuerySet, self)._clone(**kwargs)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2016-12-30 10:51
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='document',
            name='original_filename',
            field=models.CharField(blank=True, max_length=500),
        ),
        migrations.AlterField(
            model_name='document',
            name='folder',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='documents.Folder'),
        ),
    ]



from django.core.urlresolvers import reverse, reverse_lazy
from django.db import transaction
from django.db.models import F
from django.http import HttpResponse, HttpResponseRedirect, Http404
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext as _
from django.views import static
from django.views.generic import (
    CreateView,
    DeleteView,
    DetailView,
    TemplateView,
)
from django.views.generic.detail import (
    SingleObjectMixin, SingleObjectTemplateResponseMixin,
)
from django.views.generic.edit import (
    FormMixin,
    ProcessFormView,
)

from account.mixins import LoginRequiredMixin

from .conf import settings
from .forms import (
    ColleagueFolderShareForm,
    DocumentCreateForm,
    DocumentCreateFormWithName,
    FolderCreateForm
)
from .hooks import hookset
from .models import (
    Document,
    Folder,
    UserStorage,
)


class IndexView(LoginRequiredMixin, TemplateView):

    template_name = "pinax/documents/index.html"

    def get_context_data(self, **kwargs):
        ctx = kwargs
        ctx.update({
            "members": Folder.objects.members(None, user=self.request.user),
            "storage": self.request.user.storage,
            "can_share": False,
        })
        return ctx


class FolderCreate(LoginRequiredMixin, CreateView):
    model = Folder
    form_class = FolderCreateForm
    template_name = "pinax/documents/folder_create.html"
    parent = None

    def get(self, request, *args, **kwargs):
        if "p" in request.GET:
            qs = self.model.objects.for_user(request.user)
            self.parent = get_object_or_404(qs, pk=request.GET["p"])
        else:
            self.parent = None
        return super(FolderCreate, self).get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        kwargs.setdefault("parent", self.parent)
        return super(FolderCreate, self).get_context_data(**kwargs)

    def get_initial(self):
        if self.parent:
            self.initial["parent"] = self.parent
        return super(FolderCreate, self).get_initial()

    def get_form_kwargs(self):
        kwargs = super(FolderCreate, self).get_form_kwargs()
        kwargs.update({"folders": self.model.objects.for_user(self.request.user)})
        return kwargs

    def create_folder(self, **kwargs):
        folder = self.model.objects.create(**kwargs)
        folder.touch(self.request.user)
        # if folder is not amongst anything shared it will share with no
        # users which share will no-op; perhaps not the best way?
        folder.share(folder.shared_parent().shared_with())
        return folder

    def form_valid(self, form):
        kwargs = {
            "name": form.cleaned_data["name"],
            "author": self.request.user,
            "parent": form.cleaned_data["parent"],
        }
        self.object = self.create_folder(**kwargs)
        hookset.folder_created_message(self.request, self.object)
        return HttpResponseRedirect(self.get_success_url())


class FolderDetail(LoginRequiredMixin, DetailView):
    model = Folder
    template_name = "pinax/documents/folder_detail.html"

    def get_queryset(self):
        qs = super(FolderDetail, self).get_queryset()
        qs = qs.for_user(self.request.user)
        return qs

    def get_form_kwargs(self):
        kwargs = super(FolderShare, self).get_form_kwargs()
        kwargs.update({"folders": self.model.objects.for_user(self.request.user)})
        return kwargs

    def get_context_data(self, **kwargs):
        context = super(FolderDetail, self).get_context_data(**kwargs)
        ctx = {
            "members": self.object.members(user=self.request.user),
            "can_share": self.object.can_share(self.request.user),
        }
        context.update(ctx)
        return context


class FolderShare(LoginRequiredMixin,
                  SingleObjectTemplateResponseMixin,
                  FormMixin,
                  SingleObjectMixin,
                  ProcessFormView):
    model = Folder
    context_object_name = "folder"
    form_class = ColleagueFolderShareForm
    template_name = "pinax/documents/folder_share.html"

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super(FolderShare, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super(FolderShare, self).post(request, *args, **kwargs)

    def get_queryset(self):
        qs = super(FolderShare, self).get_queryset()
        qs = qs.for_user(self.request.user)
        return qs

    def get_object(self):
        folder = super(FolderShare, self).get_object()
        if not folder.can_share(self.request.user):
            raise Http404(_("Cannot share folder '{}'.".format(folder)))
        return folder

    def get_form_kwargs(self):
        kwargs = super(FolderShare, self).get_form_kwargs()
        can_share_with = hookset.share_with_options(self.request.user, self.object)
        kwargs.update({"colleagues": can_share_with})
        return kwargs

    def form_valid(self, form):
        self.object.share(form.cleaned_data["participants"])
        hookset.folder_shared_message(self.request, self.request.user, self.object)
        return HttpResponseRedirect(self.get_success_url())

    def get_success_url(self):
        return reverse("pinax_documents:folder_detail", args=[self.object.pk])

    def get_context_data(self, **kwargs):
        context = super(FolderShare, self).get_context_data(**kwargs)
        ctx = {
            "participants": self.object.shared_with(),
        }
        context.update(ctx)
        return context


class FolderDelete(LoginRequiredMixin, DeleteView):
    model = Folder
    success_url = reverse_lazy("pinax_documents:document_index")
    template_name = 'pinax/documents/folder_confirm_delete.html'

    def delete(self, request, *args, **kwargs):
        hookset.folder_pre_delete(self.request, self.get_object())
        success_url = super(FolderDelete, self).delete(request, *args, **kwargs)
        hookset.folder_deleted_message(self.request, self.object)
        return success_url


class DocumentCreate(LoginRequiredMixin, CreateView):
    model = Document
    form_class = DocumentCreateForm
    template_name = "pinax/documents/document_create.html"
    folder = None

    def get(self, request, *args, **kwargs):
        if "f" in request.GET:
            qs = Folder.objects.for_user(request.user)
            self.folder = get_object_or_404(qs, pk=request.GET["f"])
        else:
            self.folder = None
        return super(DocumentCreate, self).get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        kwargs.setdefault("folder", self.folder)
        return super(DocumentCreate, self).get_context_data(**kwargs)

    def get_initial(self):
        if self.folder:
            self.initial["folder"] = self.folder
        return super(DocumentCreate, self).get_initial()

    def get_form_kwargs(self):
        kwargs = super(DocumentCreate, self).get_form_kwargs()
        kwargs.update({"folders": Folder.objects.for_user(self.request.user),
                       "storage": self.request.user.storage})
        return kwargs

    def create_document(self, **kwargs):
        document = self.model.objects.create(**kwargs)
        document.touch(self.request.user)
        if document.folder is not None:
            # if folder is not amongst anything shared it will share with no
            # users which share will no-op; perhaps not the best way?
            document.share(document.folder.shared_parent().shared_with())
        return document

    def increase_usage(self, bytes):
        # increase usage for this user based on document size
        storage_qs = UserStorage.objects.filter(pk=self.request.user.storage.pk)
        storage_qs.update(bytes_used=F("bytes_used") + bytes)

    def get_create_kwargs(self, form):
        return {
            "name": form.cleaned_data["file"].name,
            "original_filename": form.cleaned_data["file"].name,
            "folder": form.cleaned_data["folder"],
            "author": self.request.user,
            "file": form.cleaned_data["file"],
        }

    def form_valid(self, form):
        with transaction.atomic():
            kwargs = self.get_create_kwargs(form)
            self.object = self.create_document(**kwargs)
            hookset.document_created_message(self.request, self.object)
            bytes = form.cleaned_data["file"].size
            self.increase_usage(bytes)
            return HttpResponseRedirect(self.get_success_url())


class DocumentWithCustomNameCreate(DocumentCreate):

    form_class = DocumentCreateFormWithName

    def get_create_kwargs(self, form):
        return {
            "name": form.cleaned_data["name"],
            "original_filename": form.cleaned_data["file"].name,
            "folder": form.cleaned_data["folder"],
            "author": self.request.user,
            "file": form.cleaned_data["file"],
        }


class DocumentDetail(LoginRequiredMixin, DetailView):
    model = Document
    template_name = "pinax/documents/document_detail.html"

    def get_queryset(self):
        qs = super(DocumentDetail, self).get_queryset()
        qs = qs.for_user(self.request.user)
        return qs


class DocumentDownload(LoginRequiredMixin, DetailView):
    model = Document

    def get_queryset(self):
        qs = super(DocumentDownload, self).get_queryset()
        qs = qs.for_user(self.request.user)
        return qs

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        if settings.DOCUMENTS_USE_X_ACCEL_REDIRECT:
            response = HttpResponse()
            response["X-Accel-Redirect"] = self.object.file.url
            # delete content-type to allow Gondor to determine the filetype and
            # we definitely don't want Django's crappy default :-)
            del response["content-type"]
        else:
            # Note:
            #
            # The 'django.views.static.py' docstring states:
            #
            #     Views and functions for serving static files. These are only to be used
            #     during development, and SHOULD NOT be used in a production setting.
            #
            response = static.serve(request, self.object.file.name,
                                    document_root=settings.MEDIA_ROOT)
        return response


class DocumentDelete(LoginRequiredMixin, DeleteView):
    model = Document
    success_url = reverse_lazy("pinax_documents:document_index")
    template_name = 'pinax/documents/document_confirm_delete.html'

    def delete(self, request, *args, **kwargs):
        success_url = super(DocumentDelete, self).delete(request, *args, **kwargs)
        hookset.document_deleted_message(self.request, self.object)
        return success_url



from django.template import Library

from ..utils import convert_bytes


register = Library()


@register.filter
def can_share(member, user):
    if member is None:
        return False
    return member.can_share(user)


@register.filter
def readable_bytes(bytes):
    return convert_bytes(bytes)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2016-12-30 10:55
from __future__ import unicode_literals

from django.db import migrations


def populate_original(apps, schema_editor):
    Document = apps.get_model("documents", "Document")
    db_alias = schema_editor.connection.alias
    for document in Document.objects.using(db_alias).all():
        document.original_filename = document.name
        document.save()


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '0002_auto_20161230_1051'),
    ]

    operations = [
        migrations.RunPython(populate_original)
    ]



from django import forms

try:
    from account.utils import user_display
except ImportError:
    def user_display(user):
        return user.username

from .hooks import hookset
from .models import (
    Document,
    Folder,
)


class FolderCreateForm(forms.ModelForm):

    class Meta:
        model = Folder
        fields = ["name", "parent"]
        widgets = {
            "parent": forms.HiddenInput,
        }

    def clean(self):
        name = self.cleaned_data["name"]
        parent = self.cleaned_data.get("parent")
        if Folder.already_exists(name, parent):
            raise forms.ValidationError("{} already exists.".format(name))

    def __init__(self, *args, **kwargs):
        folders = kwargs.pop("folders")
        super(FolderCreateForm, self).__init__(*args, **kwargs)
        self.fields["parent"].queryset = folders


class DocumentCreateForm(forms.ModelForm):

    class Meta:
        model = Document
        fields = ["folder", "file"]
        widgets = {
            "folder": forms.HiddenInput,
        }

    def __init__(self, *args, **kwargs):
        folders = kwargs.pop("folders")
        self.storage = kwargs.pop("storage")
        super(DocumentCreateForm, self).__init__(*args, **kwargs)
        self.fields["folder"].queryset = folders

    def clean_file(self):
        value = self.cleaned_data["file"]
        if (value.size + self.storage.bytes_used) > self.storage.bytes_total:
            raise forms.ValidationError("File will exceed storage capacity.")
        return value

    def clean(self):
        if "file" in self.cleaned_data:
            name = self.cleaned_data.get("file").name
            folder = self.cleaned_data.get("folder")
            if Document.already_exists(name, folder):
                raise forms.ValidationError(
                    hookset.already_exists_validation_message(name, folder)
                )


class DocumentCreateFormWithName(DocumentCreateForm):

    class Meta:
        model = Document
        fields = ["folder", "file", "name"]
        widgets = {
            "folder": forms.HiddenInput,
        }


class UserMultipleChoiceField(forms.ModelMultipleChoiceField):

    def label_from_instance(self, obj):
        return user_display(obj)


class FolderShareForm(forms.Form):

    participants = UserMultipleChoiceField(
        queryset=None,
        widget=forms.SelectMultiple(
            attrs={
                "class": "span6",
                "data-placeholder": "Choose participants... "
            }
        )
    )


class ColleagueFolderShareForm(FolderShareForm):

    def __init__(self, *args, **kwargs):
        colleagues = kwargs.pop("colleagues")
        super(ColleagueFolderShareForm, self).__init__(*args, **kwargs)
        self.fields["participants"].queryset = colleagues



import itertools
import math

from django.core.urlresolvers import reverse
from django.db import models
from django.db.models import F
from django.utils import timezone
from django.utils.encoding import python_2_unicode_compatible

from django.contrib.auth import get_user_model

from .compat import izip_longest
from .conf import settings
from .exceptions import DuplicateFolderNameError, DuplicateDocumentNameError
from .hooks import hookset
from .managers import FolderManager, FolderQuerySet, DocumentQuerySet


def uuid_filename(instance, filename):
    return hookset.file_upload_to(instance, filename)


@python_2_unicode_compatible
class Folder(models.Model):

    name = models.CharField(max_length=140)
    parent = models.ForeignKey("self", null=True, blank=True)
    author = models.ForeignKey(settings.AUTH_USER_MODEL, related_name="+")
    created = models.DateTimeField(default=timezone.now)
    modified = models.DateTimeField(default=timezone.now)
    modified_by = models.ForeignKey(settings.AUTH_USER_MODEL, related_name="+")

    objects = FolderManager.from_queryset(FolderQuerySet)()

    kind = "folder"
    icon = "folder-open"
    shared = None

    @classmethod
    def shared_user_model(cls):
        return FolderSharedUser

    @classmethod
    def already_exists(cls, name, parent=None):
        return cls.objects.filter(name=name, parent=parent).exists()

    def __str__(self):
        return self.name

    def save(self, **kwargs):
        if not self.pk and Folder.already_exists(self.name, self.parent):
            raise DuplicateFolderNameError("{} already exists in this folder.".format(self.name))
        self.touch(self.author, commit=False)
        super(Folder, self).save(**kwargs)

    def get_absolute_url(self):
        return reverse("pinax_documents:folder_detail", args=[self.pk])

    def unique_id(self):
        return "f-%d" % self.id

    def members(self, **kwargs):
        return Folder.objects.members(self, **kwargs)

    def touch(self, user, commit=True):
        self.modified = timezone.now()
        self.modified_by = user
        if commit:
            if self.parent:
                self.parent.touch(user)
            self.save()

    @property
    def size(self):
        """
        Return size of this folder.
        """
        return sum([m.size for m in self.members(direct=False) if m.kind == "document"])

    def breadcrumbs(self):
        """
        Produces a list of ancestors (excluding self).
        """
        crumbs = []
        if self.parent:
            crumbs.extend(self.parent.breadcrumbs())
            crumbs.append(self.parent)
        return crumbs

    def shared_queryset(self):
        """
        Returns queryset of this folder mapped into the shared user model.
        The queryset should only consist of zero or one instances (aka shared
        or not shared.) This method is mostly used for convenience.
        """
        model = self.shared_user_model()
        return model._default_manager.filter(**{model.obj_attr: self})

    @property
    def shared(self):
        """
        Determines if self is shared. This checks the denormalization and
        does not return whether self SHOULD be shared (based on parents.)
        """
        return self.shared_queryset().exists()

    def shared_ui(self):
        """
        Returns boolean based on whether self should show any shared UI.
        """
        return self.parent_id is None and self.shared

    def shared_with(self, user=None):
        """
        Returns a User queryset of users shared on this folder, or, if user
        is given optimizes the check and returns boolean.
        """
        User = get_user_model()
        qs = self.shared_queryset()
        if user is not None:
            return qs.filter(user=user).exists()
        if not qs.exists():
            return User.objects.none()
        return User.objects.filter(pk__in=qs.values("user"))

    def shared_parent(self):
        """
        Returns the folder object that is the shared parent (the root of
        a shared folder hierarchy) or None if there is no shared parent.
        """
        root = self
        a, b = itertools.tee(reversed(self.breadcrumbs()))
        next(b, None)
        for folder, parent in izip_longest(a, b):
            if folder.shared:
                root = folder
            if parent is None or not parent.shared:
                break
        return root

    def can_share(self, user):
        """
        Ensures folder is top-level and `user` is the author.
        """
        return hookset.can_share_folder(user, self)

    def share(self, users):
        """
        Ensures self is shared with given users (can accept users who are
        already shared on self).
        """
        users = [u for u in users if not self.shared_with(user=u)]
        if users:
            members = [self] + self.members(direct=False)
            FM, DM = self.shared_user_model(), Document.shared_user_model()
            fm, dm = [], []
            for member, user in itertools.product(members, users):
                if user.pk == member.author_id:
                    continue
                if isinstance(member, Folder):
                    fm.append(FM(**{FM.obj_attr: member, "user": user}))
                if isinstance(member, Document):
                    dm.append(DM(**{DM.obj_attr: member, "user": user}))
            FM._default_manager.bulk_create(fm)
            DM._default_manager.bulk_create(dm)

    def delete_url(self):
        return reverse(
            "pinax_documents:folder_delete",
            args=[self.pk]
        )


@python_2_unicode_compatible
class Document(models.Model):

    name = models.CharField(max_length=255)
    folder = models.ForeignKey(Folder, null=True, blank=True)
    author = models.ForeignKey(settings.AUTH_USER_MODEL, related_name="+")
    created = models.DateTimeField(default=timezone.now)
    modified = models.DateTimeField(default=timezone.now)
    modified_by = models.ForeignKey(settings.AUTH_USER_MODEL, related_name="+")
    file = models.FileField(upload_to=uuid_filename)
    original_filename = models.CharField(max_length=500)

    objects = DocumentQuerySet.as_manager()

    kind = "document"
    icon = "file"
    shared = None

    def delete(self, *args, **kwargs):
        bytes_to_free = self.size
        super(Document, self).delete(*args, **kwargs)
        storage_qs = UserStorage.objects.filter(pk=self.author.storage.pk)
        storage_qs.update(bytes_used=F("bytes_used") - bytes_to_free)

    @classmethod
    def shared_user_model(cls):
        return DocumentSharedUser

    @classmethod
    def already_exists(cls, name, folder=None):
        return cls.objects.filter(name=name, folder=folder).exists()

    def __str__(self):
        return self.name

    def save(self, **kwargs):
        if not self.pk and Document.already_exists(self.name, self.folder):
            raise DuplicateDocumentNameError("{} already exists in this folder.".format(self.name))
        self.touch(self.author, commit=False)
        super(Document, self).save(**kwargs)

    def get_absolute_url(self):
        return reverse("pinax_documents:document_detail", args=[self.pk])

    def unique_id(self):
        return "d-%d" % self.id

    def touch(self, user, commit=True):
        self.modified = timezone.now()
        self.modified_by = user
        if commit:
            if self.folder:
                self.folder.touch(user)
            self.save()

    @property
    def size(self):
        return self.file.size

    def breadcrumbs(self):
        crumbs = []
        if self.folder:
            crumbs.extend(self.folder.breadcrumbs())
            crumbs.append(self.folder)
        return crumbs

    def shared_queryset(self):
        """
        Returns queryset of this folder mapped into the shared user model.
        The queryset should only consist of zero or one instances (aka shared
        or not shared.) This method is mostly used for convenience.
        """
        model = self.shared_user_model()
        return model._default_manager.filter(**{model.obj_attr: self})

    @property
    def shared(self):
        """
        Determines if self is shared. This checks the denormalization and
        does not return whether self SHOULD be shared (based on parents.)
        """
        return self.shared_queryset().exists()

    def shared_ui(self):
        return False

    def shared_with(self, user=None):
        """
        Returns a User queryset of users shared on this folder, or, if user
        is given optimizes the check and returns boolean.
        """
        User = get_user_model()
        qs = self.shared_queryset()
        if user is not None:
            return qs.filter(user=user).exists()
        if not qs.exists():
            return User.objects.none()
        return User.objects.filter(pk__in=qs.values("user"))

    def share(self, users):
        users = [u for u in users if not self.shared_with(user=u)]
        if users:
            model = self.shared_user_model()
            objs = []
            for user in users:
                objs.append(self.shared_user_model()(**{model.obj_attr: self, "user": user}))
            model._default_manager.bulk_create(objs)

    def download_url(self):
        return reverse(
            "pinax_documents:document_download",
            args=[self.pk]
        )

    def delete_url(self):
        return reverse(
            "pinax_documents:document_delete",
            args=[self.pk]
        )


class MemberSharedUser(models.Model):

    user = models.ForeignKey(settings.AUTH_USER_MODEL)
    # @@@ privileges

    class Meta:
        abstract = True

    @classmethod
    def for_user(cls, user):
        qs = cls._default_manager.filter(user=user)
        return qs.values_list(cls.obj_attr, flat=True)


class FolderSharedUser(MemberSharedUser):

    folder = models.ForeignKey(Folder)
    obj_attr = "folder"

    class Meta:
        unique_together = [("folder", "user")]


class DocumentSharedUser(MemberSharedUser):

    document = models.ForeignKey(Document)
    obj_attr = "document"

    class Meta:
        unique_together = [("document", "user")]


class UserStorage(models.Model):

    user = models.OneToOneField(settings.AUTH_USER_MODEL, related_name="storage")
    bytes_used = models.BigIntegerField(default=0)
    bytes_total = models.BigIntegerField(default=0)

    @property
    def percentage(self):
        return int(math.ceil((float(self.bytes_used) / self.bytes_total) * 100))

    @property
    def color(self):
        return hookset.storage_color(self)
